@@ -423,7 +423,7 @@ def preconfigure_modules(compiler, settings):
             '',
         ] + lines + [
             'Please refer to the Installation Guide for details:',
-            'https://docs-cupy.chainer.org/en/stable/install.html',
+            'https://docs.cupy.dev/en/stable/install.html',
             '',
         ]
 
@@ -728,7 +728,7 @@ def check_extensions(extensions):
                 raise RuntimeError('''\
 Missing file: {}
 Please install Cython {} or later. Please also check the version of Cython.
-See https://docs-cupy.chainer.org/en/stable/install.html for details.
+See https://docs.cupy.dev/en/stable/install.html for details.
 '''.format(f, required_cython_version))
 
 
