@@ -39,34 +39,22 @@ class TestSort(unittest.TestCase):
 
     # Test dtypes
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
-    @testing.numpy_cupy_allclose()
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
     def test_sort_dtype(self, xp, dtype):
         a = testing.shaped_random((10,), xp, dtype)
         a.sort()
         return a
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
-    @testing.numpy_cupy_allclose()
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
     def test_external_sort_dtype(self, xp, dtype):
         a = testing.shaped_random((10,), xp, dtype)
         return xp.sort(a)
 
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_sort_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((10,), cupy, dtype)
-        with self.assertRaises(NotImplementedError):
-            a.sort()
-
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_external_sort_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((10,), cupy, dtype)
-        with self.assertRaises(NotImplementedError):
-            return cupy.sort(a)
-
     # Test contiguous arrays
 
-    @testing.numpy_cupy_allclose()
+    @testing.numpy_cupy_array_equal()
     def test_sort_contiguous(self, xp):
         a = testing.shaped_random((10,), xp)  # C contiguous view
         a.sort()
@@ -77,12 +65,12 @@ class TestSort(unittest.TestCase):
         with self.assertRaises(NotImplementedError):
             a.sort()
 
-    @testing.numpy_cupy_allclose()
+    @testing.numpy_cupy_array_equal()
     def test_external_sort_contiguous(self, xp):
         a = testing.shaped_random((10,), xp)  # C contiguous view
         return xp.sort(a)
 
-    @testing.numpy_cupy_allclose()
+    @testing.numpy_cupy_array_equal()
     def test_external_sort_non_contiguous(self, xp):
         a = testing.shaped_random((10,), xp)[::2]  # Non contiguous view
         return xp.sort(a)
@@ -90,11 +78,23 @@ class TestSort(unittest.TestCase):
     # Test axis
 
     @testing.numpy_cupy_array_equal()
-    def test_sort_axis(self, xp):
-        a = testing.shaped_random((2, 3, 3), xp)
+    def test_sort_axis1(self, xp):
+        a = testing.shaped_random((2, 3, 4), xp)
         a.sort(axis=0)
         return a
 
+    @testing.numpy_cupy_array_equal()
+    def test_sort_axis2(self, xp):
+        a = testing.shaped_random((2, 3, 4), xp)
+        a.sort(axis=1)
+        return a
+
+    @testing.numpy_cupy_array_equal()
+    def test_sort_axis3(self, xp):
+        a = testing.shaped_random((2, 3, 4), xp)
+        a.sort(axis=2)
+        return a
+
     @testing.numpy_cupy_array_equal()
     def test_external_sort_axis(self, xp):
         a = testing.shaped_random((2, 3, 3), xp)
@@ -160,6 +160,40 @@ class TestSort(unittest.TestCase):
         with self.assertRaises(cupy.core._AxisError):
             cupy.sort(a, axis=-4)
 
+    # Test NaN ordering
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan1(self, xp, dtype):
+        a = testing.shaped_random((10,), xp, dtype)
+        a[2] = a[6] = xp.nan
+        out = xp.sort(a)
+        return out
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan2(self, xp, dtype):
+        a = testing.shaped_random((2, 3, 4), xp, dtype)
+        a[0, 2, 1] = a[1, 0, 3] = xp.nan
+        out = xp.sort(a, axis=0)
+        return out
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan3(self, xp, dtype):
+        a = testing.shaped_random((2, 3, 4), xp, dtype)
+        a[0, 2, 1] = a[1, 0, 3] = xp.nan
+        out = xp.sort(a, axis=1)
+        return out
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan4(self, xp, dtype):
+        a = testing.shaped_random((2, 3, 4), xp, dtype)
+        a[0, 2, 1] = a[1, 0, 3] = xp.nan
+        out = xp.sort(a, axis=2)
+        return out
+
 
 @testing.gpu
 class TestLexsort(unittest.TestCase):
@@ -191,17 +225,51 @@ class TestLexsort(unittest.TestCase):
 
     # Test dtypes
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
-    @testing.numpy_cupy_allclose()
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
     def test_lexsort_dtype(self, xp, dtype):
         a = testing.shaped_random((2, 10), xp, dtype)
         return xp.lexsort(a)
 
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_lexsort_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((2, 10), cupy, dtype)
-        with self.assertRaises(TypeError):
-            return cupy.lexsort(a)
+    # Test NaN ordering
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan1(self, xp, dtype):
+        a = testing.shaped_random((2, 10), xp, dtype)
+        a[0, 2] = a[0, 6] = xp.nan
+        return xp.lexsort(a)
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan2(self, xp, dtype):
+        a = testing.shaped_random((2, 10), xp, dtype)
+        a[1, 2] = a[0, 6] = xp.nan
+        return xp.lexsort(a)
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan3(self, xp, dtype):
+        a = testing.shaped_random((2, 10), xp, dtype)
+        a[1, 2] = a[1, 6] = xp.nan
+        return xp.lexsort(a)
+
+    # Test non C-contiguous input
+
+    @testing.numpy_cupy_array_equal()
+    def test_view(self, xp):
+        # from #3232
+        a = testing.shaped_random((4, 8), xp, dtype=xp.float64)
+        a = a.T[::-1]
+        return xp.lexsort(a)
+
+    @testing.numpy_cupy_array_equal()
+    def test_F_order(self, xp):
+        a = testing.shaped_random((4, 8), xp, dtype=xp.float64)
+        a = xp.asfortranarray(a)
+        assert a.flags.f_contiguous
+        assert not a.flags.c_contiguous
+        return xp.lexsort(a)
 
 
 @testing.parameterize(*testing.product({
@@ -220,19 +288,19 @@ class TestArgsort(unittest.TestCase):
     # Test base cases
 
     @testing.with_requires('numpy>=1.13')
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_argsort_zero_dim(self, xp, dtype):
         a = testing.shaped_random((), xp, dtype)
         return self.argsort(a)
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_argsort_one_dim(self, xp, dtype):
         a = testing.shaped_random((10,), xp, dtype)
         return self.argsort(a)
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_argsort_multi_dim(self, xp, dtype):
         a = testing.shaped_random((2, 3, 3), xp, dtype)
@@ -243,14 +311,6 @@ class TestArgsort(unittest.TestCase):
         a = xp.array([1, 0, 2, 3])[::2]
         return self.argsort(a)
 
-    # Test unsupported dtype
-
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_argsort_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((10,), cupy, dtype)
-        with self.assertRaises(NotImplementedError):
-            return self.argsort(a)
-
     # Test axis
 
     @testing.numpy_cupy_array_equal()
@@ -317,6 +377,22 @@ class TestArgsort(unittest.TestCase):
         self.argsort(a)
         testing.assert_allclose(a, b)
 
+    # Test NaN ordering
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan1(self, xp, dtype):
+        a = testing.shaped_random((10,), xp, dtype)
+        a[2] = a[6] = xp.nan
+        return self.argsort(a)
+
+    @testing.for_dtypes('efdFD')
+    @testing.numpy_cupy_array_equal()
+    def test_nan2(self, xp, dtype):
+        a = testing.shaped_random((2, 3, 4), xp, dtype)
+        a[0, 2, 1] = a[1, 1, 3] = xp.nan
+        return self.argsort(a)
+
 
 @testing.gpu
 class TestMsort(unittest.TestCase):
@@ -330,26 +406,18 @@ class TestMsort(unittest.TestCase):
         a = testing.shaped_random((), xp)
         return xp.msort(a)
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_msort_one_dim(self, xp, dtype):
         a = testing.shaped_random((10,), xp, dtype)
         return xp.msort(a)
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=False)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_msort_multi_dim(self, xp, dtype):
         a = testing.shaped_random((2, 3), xp, dtype)
         return xp.msort(a)
 
-    # Test unsupported dtype
-
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_msort_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((10,), cupy, dtype)
-        with self.assertRaises(NotImplementedError):
-            return cupy.msort(a)
-
 
 @testing.parameterize(*testing.product({
     'external': [False, True],
@@ -376,12 +444,9 @@ class TestPartition(unittest.TestCase):
             with pytest.raises(numpy.AxisError):
                 self.partition(a, kth)
 
-    @testing.for_all_dtypes(no_complex=True)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_equal()
     def test_partition_one_dim(self, xp, dtype):
-        if self.length == 10 and dtype in [xp.float16, xp.bool_]:
-            return cupy.zeros(1)  # dummy
-
         a = testing.shaped_random((self.length,), xp, dtype)
         kth = 2
         x = self.partition(a, kth)
@@ -389,12 +454,9 @@ class TestPartition(unittest.TestCase):
         self.assertTrue(xp.all(x[kth:kth + 1] <= x[kth + 1:]))
         return x[kth]
 
-    @testing.for_all_dtypes(no_complex=True)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
     def test_partition_multi_dim(self, xp, dtype):
-        if self.length == 10 and dtype in [xp.float16, xp.bool_]:
-            return cupy.zeros(1)  # dummy
-
         a = testing.shaped_random((10, 10, self.length), xp, dtype)
         kth = 2
         x = self.partition(a, kth)
@@ -402,19 +464,6 @@ class TestPartition(unittest.TestCase):
         self.assertTrue(xp.all(x[:, :, kth:kth + 1] <= x[:, :, kth + 1:]))
         return x[:, :, kth:kth + 1]
 
-    # Test unsupported dtype
-
-    @testing.for_dtypes([numpy.float16, numpy.bool_, numpy.complex64,
-                         numpy.complex128])
-    def test_partition_unsupported_dtype(self, dtype):
-        if self.length != 10 and not cupy.issubdtype(dtype, complex):
-            return
-
-        a = testing.shaped_random((self.length,), cupy, dtype)
-        kth = 2
-        with self.assertRaises(NotImplementedError):
-            return self.partition(a, kth)
-
     # Test non-contiguous array
 
     @testing.numpy_cupy_equal()
@@ -543,7 +592,7 @@ class TestArgpartition(unittest.TestCase):
             with pytest.raises(ValueError):
                 self.argpartition(a, kth)
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=True)
+    @testing.for_all_dtypes()
     @testing.numpy_cupy_equal()
     def test_argpartition_one_dim(self, xp, dtype):
         a = testing.shaped_random((10,), xp, dtype, 100)
@@ -553,7 +602,9 @@ class TestArgpartition(unittest.TestCase):
         self.assertTrue((a[idx[kth]] < a[idx[kth + 1:]]).all())
         return idx[kth]
 
-    @testing.for_all_dtypes(no_float16=True, no_bool=True, no_complex=True)
+    # TODO(leofang): test all dtypes -- this workaround needs to be kept,
+    # likely due to #3287? Need investigation.
+    @testing.for_all_dtypes(no_bool=True)
     @testing.numpy_cupy_array_equal()
     def test_argpartition_multi_dim(self, xp, dtype):
         a = testing.shaped_random((3, 3, 10), xp, dtype, 100)
@@ -567,15 +618,6 @@ class TestArgpartition(unittest.TestCase):
                          a[rows, cols, idx[:, :, kth + 1:]]).all())
         return idx[:, :, kth:kth + 1]
 
-    # Test unsupported dtype
-
-    @testing.for_dtypes([numpy.float16, numpy.bool_])
-    def test_argpartition_unsupported_dtype(self, dtype):
-        a = testing.shaped_random((10,), cupy, dtype, 100)
-        kth = 2
-        with self.assertRaises(NotImplementedError):
-            return self.argpartition(a, kth)
-
     # Test non-contiguous array
 
     @testing.numpy_cupy_equal()
